from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="Identity")


@_attrs_define
class Identity:
    """Identity model

    Attributes:
        fqda (str):
        primary_id (str):
        user_name (str):
    """

    fqda: str
    primary_id: str
    user_name: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        fqda = self.fqda
        primary_id = self.primary_id
        user_name = self.user_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "fqda": fqda,
                "primary_id": primary_id,
                "user_name": user_name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`Identity` from a dict"""
        d = src_dict.copy()
        fqda = d.pop("fqda")

        primary_id = d.pop("primary_id")

        user_name = d.pop("user_name")

        identity = cls(
            fqda=fqda,
            primary_id=primary_id,
            user_name=user_name,
        )

        return identity



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="GroupCreateGroupBody")


@_attrs_define
class GroupCreateGroupBody:
    """GroupCreateGroupBody model

    Attributes:
        group_name (str):
    """

    group_name: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        group_name = self.group_name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "group_name": group_name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`GroupCreateGroupBody` from a dict"""
        d = src_dict.copy()
        group_name = d.pop("group_name")

        group_create_group_body = cls(
            group_name=group_name,
        )

        return group_create_group_body



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="CollectionCdeStatsItemDistribItem")


@_attrs_define
class CollectionCdeStatsItemDistribItem:
    """CollectionCdeStatsItemDistribItem model

    Attributes:
        having_num_cdes (int):
        num_items (int):
    """

    having_num_cdes: int
    num_items: int

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        having_num_cdes = self.having_num_cdes
        num_items = self.num_items

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "havingNumCdes": having_num_cdes,
                "numItems": num_items,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionCdeStatsItemDistribItem` from a dict"""
        d = src_dict.copy()
        having_num_cdes = d.pop("havingNumCdes")

        num_items = d.pop("numItems")

        collection_cde_stats_item_distrib_item = cls(
            having_num_cdes=having_num_cdes,
            num_items=num_items,
        )

        return collection_cde_stats_item_distrib_item



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.dictionary_search_options_options import DictionarySearchOptionsOptions
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="DictionarySearchOptions")


@_attrs_define
class DictionarySearchOptions:
    """DictionarySearchOptions model

    Attributes:
        source (str):
        options (Union[Unset, DictionarySearchOptionsOptions]):
    """

    source: str
    options: Union[Unset, "DictionarySearchOptionsOptions"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        source = self.source
        options: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.options, Unset):
            options = self.options.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "source": source,
            }
        )
        if options is not UNSET:
            field_dict["options"] = options

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`DictionarySearchOptions` from a dict"""
        d = src_dict.copy()
        source = d.pop("source")

        _options = d.pop("options", UNSET)
        options: Union[Unset, DictionarySearchOptionsOptions]
        if isinstance(_options, Unset):
            options = UNSET
        else:
            options = DictionarySearchOptionsOptions.from_dict(_options)

        dictionary_search_options = cls(
            source=source,
            options=options,
        )

        return dictionary_search_options



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="DataElementPermissibleValuesTextRangeTextRange")


@_attrs_define
class DataElementPermissibleValuesTextRangeTextRange:
    """Controls on the textual domain of the data.

    Attributes:
        max_length (Union[Unset, int]):
        meaning (Union[Unset, str]): A description of this range's meaning.
        min_length (Union[Unset, int]):
        regex (Union[Unset, str]): An optional regular expression that validates the data.
    """

    max_length: Union[Unset, int] = UNSET
    meaning: Union[Unset, str] = UNSET
    min_length: Union[Unset, int] = UNSET
    regex: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        max_length = self.max_length
        meaning = self.meaning
        min_length = self.min_length
        regex = self.regex

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if max_length is not UNSET:
            field_dict["maxLength"] = max_length
        if meaning is not UNSET:
            field_dict["meaning"] = meaning
        if min_length is not UNSET:
            field_dict["minLength"] = min_length
        if regex is not UNSET:
            field_dict["regex"] = regex

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`DataElementPermissibleValuesTextRangeTextRange` from a dict"""
        d = src_dict.copy()
        max_length = d.pop("maxLength", UNSET)

        meaning = d.pop("meaning", UNSET)

        min_length = d.pop("minLength", UNSET)

        regex = d.pop("regex", UNSET)

        data_element_permissible_values_text_range_text_range = cls(
            max_length=max_length,
            meaning=meaning,
            min_length=min_length,
            regex=regex,
        )

        return data_element_permissible_values_text_range_text_range



from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.task_def import TaskDef
from ..models.task_schedule_def_frequency_type_0 import TaskScheduleDefFrequencyType0
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="TaskScheduleDef")


@_attrs_define
class TaskScheduleDef:
    """TaskScheduleDef model

    Attributes:
        name (str):
        task (TaskDef):
        cronspec (Union[None, Unset, str]):
        frequency (Union[None, TaskScheduleDefFrequencyType0, Unset]):
    """

    name: str
    task: "TaskDef"
    cronspec: Union[None, Unset, str] = UNSET
    frequency: Union[None, TaskScheduleDefFrequencyType0, Unset] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        name = self.name
        task = self.task.to_dict()
        cronspec: Union[None, Unset, str]
        if isinstance(self.cronspec, Unset):
            cronspec = UNSET
        else:
            cronspec = self.cronspec
        frequency: Union[None, Unset, str]
        if isinstance(self.frequency, Unset):
            frequency = UNSET
        elif isinstance(self.frequency, TaskScheduleDefFrequencyType0):
            frequency = self.frequency.value
        else:
            frequency = self.frequency

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "task": task,
            }
        )
        if cronspec is not UNSET:
            field_dict["cronspec"] = cronspec
        if frequency is not UNSET:
            field_dict["frequency"] = frequency

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`TaskScheduleDef` from a dict"""
        d = src_dict.copy()
        name = d.pop("name")

        task = TaskDef.from_dict(d.pop("task"))

        def _parse_cronspec(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        cronspec = _parse_cronspec(d.pop("cronspec", UNSET))

        def _parse_frequency(
            data: object,
        ) -> Union[None, TaskScheduleDefFrequencyType0, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                frequency_type_0 = TaskScheduleDefFrequencyType0(data)

                return frequency_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, TaskScheduleDefFrequencyType0, Unset], data)

        frequency = _parse_frequency(d.pop("frequency", UNSET))

        task_schedule_def = cls(
            name=name,
            task=task,
            cronspec=cronspec,
            frequency=frequency,
        )

        task_schedule_def.additional_properties = d
        return task_schedule_def

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define


T = TypeVar("T", bound="OauthProvider")


@_attrs_define
class OauthProvider:
    """OauthProvider model

    Attributes:
        callback_url (str):
        id (str):
        name (str):
        signin_url (str):
        type (str):
    """

    callback_url: str
    id: str
    name: str
    signin_url: str
    type: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        callback_url = self.callback_url
        id = self.id
        name = self.name
        signin_url = self.signin_url
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "callbackUrl": callback_url,
                "id": id,
                "name": name,
                "signinUrl": signin_url,
                "type": type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`OauthProvider` from a dict"""
        d = src_dict.copy()
        callback_url = d.pop("callbackUrl")

        id = d.pop("id")

        name = d.pop("name")

        signin_url = d.pop("signinUrl")

        type = d.pop("type")

        oauth_provider = cls(
            callback_url=callback_url,
            id=id,
            name=name,
            signin_url=signin_url,
            type=type,
        )

        return oauth_provider



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.item_status_detail import ItemStatusDetail
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="ItemStatusDetails")


@_attrs_define
class ItemStatusDetails:
    """ItemStatusDetails model

    Attributes:
        parser (Union[Unset, ItemStatusDetail]):
        storage (Union[Unset, ItemStatusDetail]):
        validator (Union[Unset, ItemStatusDetail]):
    """

    parser: Union[Unset, "ItemStatusDetail"] = UNSET
    storage: Union[Unset, "ItemStatusDetail"] = UNSET
    validator: Union[Unset, "ItemStatusDetail"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        parser: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.parser, Unset):
            parser = self.parser.to_dict()
        storage: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.storage, Unset):
            storage = self.storage.to_dict()
        validator: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.validator, Unset):
            validator = self.validator.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if parser is not UNSET:
            field_dict["parser"] = parser
        if storage is not UNSET:
            field_dict["storage"] = storage
        if validator is not UNSET:
            field_dict["validator"] = validator

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`ItemStatusDetails` from a dict"""
        d = src_dict.copy()
        _parser = d.pop("parser", UNSET)
        parser: Union[Unset, ItemStatusDetail]
        if isinstance(_parser, Unset):
            parser = UNSET
        else:
            parser = ItemStatusDetail.from_dict(_parser)

        _storage = d.pop("storage", UNSET)
        storage: Union[Unset, ItemStatusDetail]
        if isinstance(_storage, Unset):
            storage = UNSET
        else:
            storage = ItemStatusDetail.from_dict(_storage)

        _validator = d.pop("validator", UNSET)
        validator: Union[Unset, ItemStatusDetail]
        if isinstance(_validator, Unset):
            validator = UNSET
        else:
            validator = ItemStatusDetail.from_dict(_validator)

        item_status_details = cls(
            parser=parser,
            storage=storage,
            validator=validator,
        )

        return item_status_details



import datetime
from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define
from dateutil.parser import isoparse


T = TypeVar("T", bound="Account")


@_attrs_define
class Account:
    """Account model

    Attributes:
        account_id (str):
        created_on (datetime.datetime):
        is_personal (bool):
        name (str):
    """

    account_id: str
    created_on: datetime.datetime
    is_personal: bool
    name: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        account_id = self.account_id
        created_on = self.created_on.isoformat()
        is_personal = self.is_personal
        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "account_id": account_id,
                "created_on": created_on,
                "is_personal": is_personal,
                "name": name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`Account` from a dict"""
        d = src_dict.copy()
        account_id = d.pop("account_id")

        created_on = isoparse(d.pop("created_on"))

        is_personal = d.pop("is_personal")

        name = d.pop("name")

        account = cls(
            account_id=account_id,
            created_on=created_on,
            is_personal=is_personal,
            name=name,
        )

        return account



"""List users"""
from http import HTTPStatus
from typing import Any
from typing import Dict
from typing import Union

import httpx

from ... import errors
from ...client import AuthenticatedClient
from ...client import Client
from ...models.server_error import ServerError
from ...models.user_get_users_response_200 import UserGetUsersResponse200
from ...types import Response
from ...types import UNSET
from ...types import Unset


def _get_kwargs(
    *,
    groups: Union[Unset, bool] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["groups"] = groups

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/user",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Union[ServerError, UserGetUsersResponse200]:
    if response.status_code == HTTPStatus.OK:
        response_200 = UserGetUsersResponse200.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = ServerError.from_dict(response.json())

        return response_500

    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[ServerError, UserGetUsersResponse200]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    groups: Union[Unset, bool] = UNSET,
) -> Response[Union[ServerError, UserGetUsersResponse200]]:
    """List users

    List all users that are visible to the current user,
    whether logged in or not.'

    Args:
        groups (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ServerError, UserGetUsersResponse200]]
    """

    kwargs = _get_kwargs(
        groups=groups,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    groups: Union[Unset, bool] = UNSET,
) -> Union[UserGetUsersResponse200]:
    """List users

    List all users that are visible to the current user,
    whether logged in or not.'

    Args:
        groups (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[UserGetUsersResponse200]
    """

    response = sync_detailed(
        client=client,
        groups=groups,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    groups: Union[Unset, bool] = UNSET,
) -> Response[Union[ServerError, UserGetUsersResponse200]]:
    """List users

    List all users that are visible to the current user,
    whether logged in or not.'

    Args:
        groups (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[ServerError, UserGetUsersResponse200]]
    """

    kwargs = _get_kwargs(
        groups=groups,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    groups: Union[Unset, bool] = UNSET,
) -> Union[UserGetUsersResponse200]:
    """List users

    List all users that are visible to the current user,
    whether logged in or not.'

    Args:
        groups (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[UserGetUsersResponse200]
    """

    response = await asyncio_detailed(
        client=client,
        groups=groups,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.system_configuration_authentication_additional_property_type_2_type import (
    SystemConfigurationAuthenticationAdditionalPropertyType2Type,
)
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="SystemConfigurationAuthenticationAdditionalPropertyType2")


@_attrs_define
class SystemConfigurationAuthenticationAdditionalPropertyType2:
    """SystemConfigurationAuthenticationAdditionalPropertyType2 model

    Attributes:
        client_id (str):
        client_secret (str):
        enabled (Union[Unset, bool]):  Default: False.
        type (Union[Unset, SystemConfigurationAuthenticationAdditionalPropertyType2Type]):
    """

    client_id: str
    client_secret: str
    enabled: Union[Unset, bool] = False
    type: Union[
        Unset, SystemConfigurationAuthenticationAdditionalPropertyType2Type
    ] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        client_id = self.client_id
        client_secret = self.client_secret
        enabled = self.enabled
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "client_id": client_id,
                "client_secret": client_secret,
            }
        )
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SystemConfigurationAuthenticationAdditionalPropertyType2` from a dict"""
        d = src_dict.copy()
        client_id = d.pop("client_id")

        client_secret = d.pop("client_secret")

        enabled = d.pop("enabled", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, SystemConfigurationAuthenticationAdditionalPropertyType2Type]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = SystemConfigurationAuthenticationAdditionalPropertyType2Type(_type)

        system_configuration_authentication_additional_property_type_2 = cls(
            client_id=client_id,
            client_secret=client_secret,
            enabled=enabled,
            type=type,
        )

        system_configuration_authentication_additional_property_type_2.additional_properties = (
            d
        )
        return system_configuration_authentication_additional_property_type_2

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""Update a single item's metadata"""
from http import HTTPStatus
from typing import Any
from typing import Dict
from typing import Union

import httpx

from ... import errors
from ...client import AuthenticatedClient
from ...client import Client
from ...models.collections_items_put_item_body import CollectionsItemsPutItemBody
from ...models.item import Item
from ...models.server_error import ServerError
from ...types import Response


def _get_kwargs(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    body: CollectionsItemsPutItemBody,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "put",
        "url": "/collections/{collection_id}/{version_id}/items/{item_id}".format(
            collection_id=collection_id,
            version_id=version_id,
            item_id=item_id,
        ),
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Union[Item, ServerError]:
    if response.status_code == HTTPStatus.OK:
        response_200 = Item.from_dict(response.json())

        return response_200
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = ServerError.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = ServerError.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = ServerError.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.NOT_FOUND:
        response_404 = ServerError.from_dict(response.json())

        return response_404
    if response.status_code == HTTPStatus.UNSUPPORTED_MEDIA_TYPE:
        response_415 = ServerError.from_dict(response.json())

        return response_415
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = ServerError.from_dict(response.json())

        return response_500

    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Item, ServerError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
    body: CollectionsItemsPutItemBody,
) -> Response[Union[Item, ServerError]]:
    """Update a single item's metadata

    Update the item definition, such as its name, folder ID,
    metadata, storage, and parsing settings.

    This method can be used to move items to new folders, rename
    items, add or remove parsing settings, or update link targets.
    It cannot be used to change an item's type, alter its status,
    or move it to a new datastore. Performing those actions will
    require deleting and recreating the item.

    Changing the item's contents can be done using the
    `.../{item_id}/data` API endpoints.

    Args:
        collection_id (str):
        version_id (str):
        item_id (str):
        body (CollectionsItemsPutItemBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Item, ServerError]]
    """

    kwargs = _get_kwargs(
        collection_id=collection_id,
        version_id=version_id,
        item_id=item_id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
    body: CollectionsItemsPutItemBody,
) -> Union[Item]:
    """Update a single item's metadata

    Update the item definition, such as its name, folder ID,
    metadata, storage, and parsing settings.

    This method can be used to move items to new folders, rename
    items, add or remove parsing settings, or update link targets.
    It cannot be used to change an item's type, alter its status,
    or move it to a new datastore. Performing those actions will
    require deleting and recreating the item.

    Changing the item's contents can be done using the
    `.../{item_id}/data` API endpoints.

    Args:
        collection_id (str):
        version_id (str):
        item_id (str):
        body (CollectionsItemsPutItemBody):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Item]
    """

    response = sync_detailed(
        collection_id=collection_id,
        version_id=version_id,
        item_id=item_id,
        client=client,
        body=body,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed


async def asyncio_detailed(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
    body: CollectionsItemsPutItemBody,
) -> Response[Union[Item, ServerError]]:
    """Update a single item's metadata

    Update the item definition, such as its name, folder ID,
    metadata, storage, and parsing settings.

    This method can be used to move items to new folders, rename
    items, add or remove parsing settings, or update link targets.
    It cannot be used to change an item's type, alter its status,
    or move it to a new datastore. Performing those actions will
    require deleting and recreating the item.

    Changing the item's contents can be done using the
    `.../{item_id}/data` API endpoints.

    Args:
        collection_id (str):
        version_id (str):
        item_id (str):
        body (CollectionsItemsPutItemBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Item, ServerError]]
    """

    kwargs = _get_kwargs(
        collection_id=collection_id,
        version_id=version_id,
        item_id=item_id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    collection_id: str,
    version_id: str,
    item_id: str,
    *,
    client: AuthenticatedClient,
    body: CollectionsItemsPutItemBody,
) -> Union[Item]:
    """Update a single item's metadata

    Update the item definition, such as its name, folder ID,
    metadata, storage, and parsing settings.

    This method can be used to move items to new folders, rename
    items, add or remove parsing settings, or update link targets.
    It cannot be used to change an item's type, alter its status,
    or move it to a new datastore. Performing those actions will
    require deleting and recreating the item.

    Changing the item's contents can be done using the
    `.../{item_id}/data` API endpoints.

    Args:
        collection_id (str):
        version_id (str):
        item_id (str):
        body (CollectionsItemsPutItemBody):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Item]
    """

    response = await asyncio_detailed(
        collection_id=collection_id,
        version_id=version_id,
        item_id=item_id,
        client=client,
        body=body,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed



import datetime
from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.federation_address_or_object_type_2 import FederationAddressOrObjectType2
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="FederationActivity")


@_attrs_define
class FederationActivity:
    """FederationActivity model

    Attributes:
        actor (Union['FederationAddressOrObjectType2', List[str], Unset, str]):
        bcc (Union[List[str], Unset, str]):
        bto (Union[List[str], Unset, str]):
        cc (Union[List[str], Unset, str]):
        content (Union[Unset, str]):
        name (Union[Unset, str]):
        object_ (Union['FederationAddressOrObjectType2', List[str], Unset, str]):
        published (Union[Unset, datetime.datetime]):
        target (Union['FederationAddressOrObjectType2', List[str], Unset, str]):
        to (Union[List[str], Unset, str]):
        type (Union[Unset, str]):
    """

    actor: Union["FederationAddressOrObjectType2", List[str], Unset, str] = UNSET
    bcc: Union[List[str], Unset, str] = UNSET
    bto: Union[List[str], Unset, str] = UNSET
    cc: Union[List[str], Unset, str] = UNSET
    content: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    object_: Union["FederationAddressOrObjectType2", List[str], Unset, str] = UNSET
    published: Union[Unset, datetime.datetime] = UNSET
    target: Union["FederationAddressOrObjectType2", List[str], Unset, str] = UNSET
    to: Union[List[str], Unset, str] = UNSET
    type: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        actor: Union[Dict[str, Any], List[str], Unset, str]
        if isinstance(self.actor, Unset):
            actor = UNSET
        elif isinstance(self.actor, list):
            actor = self.actor

        elif isinstance(self.actor, FederationAddressOrObjectType2):
            actor = self.actor.to_dict()
        else:
            actor = self.actor
        bcc: Union[List[str], Unset, str]
        if isinstance(self.bcc, Unset):
            bcc = UNSET
        elif isinstance(self.bcc, list):
            bcc = self.bcc

        else:
            bcc = self.bcc
        bto: Union[List[str], Unset, str]
        if isinstance(self.bto, Unset):
            bto = UNSET
        elif isinstance(self.bto, list):
            bto = self.bto

        else:
            bto = self.bto
        cc: Union[List[str], Unset, str]
        if isinstance(self.cc, Unset):
            cc = UNSET
        elif isinstance(self.cc, list):
            cc = self.cc

        else:
            cc = self.cc
        content = self.content
        name = self.name
        object_: Union[Dict[str, Any], List[str], Unset, str]
        if isinstance(self.object_, Unset):
            object_ = UNSET
        elif isinstance(self.object_, list):
            object_ = self.object_

        elif isinstance(self.object_, FederationAddressOrObjectType2):
            object_ = self.object_.to_dict()
        else:
            object_ = self.object_
        published: Union[Unset, str] = UNSET
        if not isinstance(self.published, Unset):
            published = self.published.isoformat()
        target: Union[Dict[str, Any], List[str], Unset, str]
        if isinstance(self.target, Unset):
            target = UNSET
        elif isinstance(self.target, list):
            target = self.target

        elif isinstance(self.target, FederationAddressOrObjectType2):
            target = self.target.to_dict()
        else:
            target = self.target
        to: Union[List[str], Unset, str]
        if isinstance(self.to, Unset):
            to = UNSET
        elif isinstance(self.to, list):
            to = self.to

        else:
            to = self.to
        type = self.type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if actor is not UNSET:
            field_dict["actor"] = actor
        if bcc is not UNSET:
            field_dict["bcc"] = bcc
        if bto is not UNSET:
            field_dict["bto"] = bto
        if cc is not UNSET:
            field_dict["cc"] = cc
        if content is not UNSET:
            field_dict["content"] = content
        if name is not UNSET:
            field_dict["name"] = name
        if object_ is not UNSET:
            field_dict["object"] = object_
        if published is not UNSET:
            field_dict["published"] = published
        if target is not UNSET:
            field_dict["target"] = target
        if to is not UNSET:
            field_dict["to"] = to
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`FederationActivity` from a dict"""
        d = src_dict.copy()

        def _parse_actor(
            data: object,
        ) -> Union["FederationAddressOrObjectType2", List[str], Unset, str]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                componentsschemasfederation_address_or_object_type_1 = cast(
                    List[str], data
                )

                return componentsschemasfederation_address_or_object_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemasfederation_address_or_object_type_2 = (
                    FederationAddressOrObjectType2.from_dict(data)
                )

                return componentsschemasfederation_address_or_object_type_2
            except:  # noqa: E722
                pass
            return cast(
                Union["FederationAddressOrObjectType2", List[str], Unset, str], data
            )

        actor = _parse_actor(d.pop("actor", UNSET))

        def _parse_bcc(data: object) -> Union[List[str], Unset, str]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                componentsschemasfederation_address_type_1 = cast(List[str], data)

                return componentsschemasfederation_address_type_1
            except:  # noqa: E722
                pass
            return cast(Union[List[str], Unset, str], data)

        bcc = _parse_bcc(d.pop("bcc", UNSET))

        def _parse_bto(data: object) -> Union[List[str], Unset, str]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                componentsschemasfederation_address_type_1 = cast(List[str], data)

                return componentsschemasfederation_address_type_1
            except:  # noqa: E722
                pass
            return cast(Union[List[str], Unset, str], data)

        bto = _parse_bto(d.pop("bto", UNSET))

        def _parse_cc(data: object) -> Union[List[str], Unset, str]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                componentsschemasfederation_address_type_1 = cast(List[str], data)

                return componentsschemasfederation_address_type_1
            except:  # noqa: E722
                pass
            return cast(Union[List[str], Unset, str], data)

        cc = _parse_cc(d.pop("cc", UNSET))

        content = d.pop("content", UNSET)

        name = d.pop("name", UNSET)

        def _parse_object_(
            data: object,
        ) -> Union["FederationAddressOrObjectType2", List[str], Unset, str]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                componentsschemasfederation_address_or_object_type_1 = cast(
                    List[str], data
                )

                return componentsschemasfederation_address_or_object_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemasfederation_address_or_object_type_2 = (
                    FederationAddressOrObjectType2.from_dict(data)
                )

                return componentsschemasfederation_address_or_object_type_2
            except:  # noqa: E722
                pass
            return cast(
                Union["FederationAddressOrObjectType2", List[str], Unset, str], data
            )

        object_ = _parse_object_(d.pop("object", UNSET))

        _published = d.pop("published", UNSET)
        published: Union[Unset, datetime.datetime]
        if isinstance(_published, Unset):
            published = UNSET
        else:
            published = isoparse(_published)

        def _parse_target(
            data: object,
        ) -> Union["FederationAddressOrObjectType2", List[str], Unset, str]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                componentsschemasfederation_address_or_object_type_1 = cast(
                    List[str], data
                )

                return componentsschemasfederation_address_or_object_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemasfederation_address_or_object_type_2 = (
                    FederationAddressOrObjectType2.from_dict(data)
                )

                return componentsschemasfederation_address_or_object_type_2
            except:  # noqa: E722
                pass
            return cast(
                Union["FederationAddressOrObjectType2", List[str], Unset, str], data
            )

        target = _parse_target(d.pop("target", UNSET))

        def _parse_to(data: object) -> Union[List[str], Unset, str]:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                componentsschemasfederation_address_type_1 = cast(List[str], data)

                return componentsschemasfederation_address_type_1
            except:  # noqa: E722
                pass
            return cast(Union[List[str], Unset, str], data)

        to = _parse_to(d.pop("to", UNSET))

        type = d.pop("type", UNSET)

        federation_activity = cls(
            actor=actor,
            bcc=bcc,
            bto=bto,
            cc=cc,
            content=content,
            name=name,
            object_=object_,
            published=published,
            target=target,
            to=to,
            type=type,
        )

        federation_activity.additional_properties = d
        return federation_activity

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.federation_address_or_object_type_2 import FederationAddressOrObjectType2
from ..models.federation_collection_page_type import FederationCollectionPageType
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="FederationCollectionPage")


@_attrs_define
class FederationCollectionPage:
    """FederationCollectionPage model

    Attributes:
        current (Union[Unset, str]):
        first (Union[Unset, str]):
        last (Union[Unset, str]):
        next_ (Union[Unset, str]):
        ordered_items (Union[Unset, List[Union['FederationAddressOrObjectType2', List[str], str]]]):
        prev (Union[Unset, str]):
        total_items (Union[Unset, int]):
        type (Union[Unset, FederationCollectionPageType]):
    """

    current: Union[Unset, str] = UNSET
    first: Union[Unset, str] = UNSET
    last: Union[Unset, str] = UNSET
    next_: Union[Unset, str] = UNSET
    ordered_items: Union[
        Unset, List[Union["FederationAddressOrObjectType2", List[str], str]]
    ] = UNSET
    prev: Union[Unset, str] = UNSET
    total_items: Union[Unset, int] = UNSET
    type: Union[Unset, FederationCollectionPageType] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        current = self.current
        first = self.first
        last = self.last
        next_ = self.next_
        ordered_items: Union[Unset, List[Union[Dict[str, Any], List[str], str]]] = UNSET
        if not isinstance(self.ordered_items, Unset):
            ordered_items = []
            for ordered_items_item_data in self.ordered_items:
                ordered_items_item: Union[Dict[str, Any], List[str], str]
                if isinstance(ordered_items_item_data, list):
                    ordered_items_item = ordered_items_item_data

                elif isinstance(
                    ordered_items_item_data, FederationAddressOrObjectType2
                ):
                    ordered_items_item = ordered_items_item_data.to_dict()
                else:
                    ordered_items_item = ordered_items_item_data
                ordered_items.append(ordered_items_item)

        prev = self.prev
        total_items = self.total_items
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if current is not UNSET:
            field_dict["current"] = current
        if first is not UNSET:
            field_dict["first"] = first
        if last is not UNSET:
            field_dict["last"] = last
        if next_ is not UNSET:
            field_dict["next"] = next_
        if ordered_items is not UNSET:
            field_dict["orderedItems"] = ordered_items
        if prev is not UNSET:
            field_dict["prev"] = prev
        if total_items is not UNSET:
            field_dict["totalItems"] = total_items
        if type is not UNSET:
            field_dict["type"] = type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`FederationCollectionPage` from a dict"""
        d = src_dict.copy()
        current = d.pop("current", UNSET)

        first = d.pop("first", UNSET)

        last = d.pop("last", UNSET)

        next_ = d.pop("next", UNSET)

        ordered_items = []
        _ordered_items = d.pop("orderedItems", UNSET)
        for ordered_items_item_data in _ordered_items or []:

            def _parse_ordered_items_item(
                data: object,
            ) -> Union["FederationAddressOrObjectType2", List[str], str]:
                try:
                    if not isinstance(data, list):
                        raise TypeError()
                    componentsschemasfederation_address_or_object_type_1 = cast(
                        List[str], data
                    )

                    return componentsschemasfederation_address_or_object_type_1
                except:  # noqa: E722
                    pass
                try:
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemasfederation_address_or_object_type_2 = (
                        FederationAddressOrObjectType2.from_dict(data)
                    )

                    return componentsschemasfederation_address_or_object_type_2
                except:  # noqa: E722
                    pass
                return cast(
                    Union["FederationAddressOrObjectType2", List[str], str], data
                )

            ordered_items_item = _parse_ordered_items_item(ordered_items_item_data)

            ordered_items.append(ordered_items_item)

        prev = d.pop("prev", UNSET)

        total_items = d.pop("totalItems", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, FederationCollectionPageType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = FederationCollectionPageType(_type)

        federation_collection_page = cls(
            current=current,
            first=first,
            last=last,
            next_=next_,
            ordered_items=ordered_items,
            prev=prev,
            total_items=total_items,
            type=type,
        )

        federation_collection_page.additional_properties = d
        return federation_collection_page

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from pypigeon import item_io
from pypigeon.pigeon_core.api.data_dictionaries import (
    collections_dictionaries_get_ordered_dictionary,
)
from pypigeon.pigeon_core.api.data_dictionaries import (
    collections_dictionaries_new_bundle,
)
from pypigeon.pigeon_core.api.data_dictionaries import (
    collections_dictionaries_set_order,
)
from pypigeon.pigeon_core.models import BundleDefinition
from pypigeon.pigeon_core.models import DataElementBundle
from pypigeon.pigeon_core.models import OrderedDictionary
from pypigeon.pigeon_core.models import PreferredOrder
from pypigeon.pigeon_core.paginator import Paginator

from .base_commands import BaseCommands


class PddCommands(BaseCommands):
    """Operations on PDDs (data dictionaries)"""

    @BaseCommands._with_arg("collection_id")
    @BaseCommands._with_arg("filename")
    @BaseCommands._with_arg("--item-name")
    @BaseCommands._with_arg("--folder-id", default="ROOT")
    def upload(self) -> None:
        """Upload a PDD from a local file."""

        item_name = self.args.item_name or self.args.filename
        writer = item_io.PigeonItemCreatorText(
            self.args.collection_id,
            "LIVE",
            item_name,
            self.core,
            folder_id=self.args.folder_id,
            type="dictionary",
        )
        with writer as ofp:
            with open(self.args.filename) as ifp:
                ofp.write(ifp.read())

    @BaseCommands._with_arg("collection_name")
    @BaseCommands._with_arg("item_name")
    @BaseCommands._with_arg("--version-id", default="LIVE")
    def list(self) -> None:
        """List ordered elements in a PDD."""

        collection = self.client.get_collection_by_name(self.args.collection_name)
        item = collection[self.args.item_name]

        pager = Paginator[OrderedDictionary](
            collections_dictionaries_get_ordered_dictionary, self.core
        )

        for page in pager.paginate(collection.id, collection.version_id, item.id):
            for entry in page.entries:
                if isinstance(entry, DataElementBundle):
                    print(entry.name)
                    for element in entry.elements:
                        print(f"   [{element.id}] {element.name}")
                else:
                    print(f"[{entry.id}] {entry.name}")

    @BaseCommands._with_arg("collection_name")
    @BaseCommands._with_arg("item_name")
    @BaseCommands._with_arg("bundle_name")
    @BaseCommands._with_arg("element_id", nargs="+")
    def bundle(self) -> None:
        """Create a new element bundle."""

        collection = self.client.get_collection_by_name(self.args.collection_name)
        item = collection[self.args.item_name]

        print("New bundle name:", self.args.bundle_name)
        print("Element IDs:", self.args.element_id)

        new_bundle = BundleDefinition(
            name=self.args.bundle_name,
            elements=self.args.element_id,
        )
        bundle = collections_dictionaries_new_bundle.sync(
            collection.id,
            collection.version_id,
            item.id,
            body=new_bundle,
            client=self.core,
        )
        print()
        print("New bundle ID:", bundle.id)

    @BaseCommands._with_arg("collection_name")
    @BaseCommands._with_arg("item_name")
    @BaseCommands._with_arg("ids", nargs="+")
    def set_order(self) -> None:
        """Set the preferred order of the PDD."""

        collection = self.client.get_collection_by_name(self.args.collection_name)
        item = collection[self.args.item_name]

        collections_dictionaries_set_order.sync(
            collection.id,
            collection.version_id,
            item.id,
            body=PreferredOrder(order=self.args.ids),
            client=self.core,
        )



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.item_parser_options import ItemParserOptions
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="ItemParser")


@_attrs_define
class ItemParser:
    """ItemParser model

    Example:

    >>> model = ItemParser.from_dict({'name': 'auto'})

    Attributes:
        name (str):
        options (Union[Unset, ItemParserOptions]):
    """

    name: str
    options: Union[Unset, "ItemParserOptions"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        name = self.name
        options: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.options, Unset):
            options = self.options.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
            }
        )
        if options is not UNSET:
            field_dict["options"] = options

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`ItemParser` from a dict"""
        d = src_dict.copy()
        name = d.pop("name")

        _options = d.pop("options", UNSET)
        options: Union[Unset, ItemParserOptions]
        if isinstance(_options, Unset):
            options = UNSET
        else:
            options = ItemParserOptions.from_dict(_options)

        item_parser = cls(
            name=name,
            options=options,
        )

        return item_parser



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.search_dictionaries_inverse_result import SearchDictionariesInverseResult


T = TypeVar("T", bound="SearchDictionariesByItemInverseResponse")


@_attrs_define
class SearchDictionariesByItemInverseResponse:
    """SearchDictionariesByItemInverseResponse model

    Attributes:
        results (List['SearchDictionariesInverseResult']):
    """

    results: List["SearchDictionariesInverseResult"]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        results = []
        for results_item_data in self.results:
            results_item = results_item_data.to_dict()
            results.append(results_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "results": results,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`SearchDictionariesByItemInverseResponse` from a dict"""
        d = src_dict.copy()
        results = []
        _results = d.pop("results")
        for results_item_data in _results:
            results_item = SearchDictionariesInverseResult.from_dict(results_item_data)

            results.append(results_item)

        search_dictionaries_by_item_inverse_response = cls(
            results=results,
        )

        return search_dictionaries_by_item_inverse_response



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.item_type import ItemType
from ..models.parser_options import ParserOptions
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="Parser")


@_attrs_define
class Parser:
    """Parser definition

    Attributes:
        item_types (List[ItemType]):
        name (str):
        options (Union[Unset, ParserOptions]):
    """

    item_types: List[ItemType]
    name: str
    options: Union[Unset, "ParserOptions"] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        item_types = []
        for item_types_item_data in self.item_types:
            item_types_item = item_types_item_data.value
            item_types.append(item_types_item)

        name = self.name
        options: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.options, Unset):
            options = self.options.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "item_types": item_types,
                "name": name,
            }
        )
        if options is not UNSET:
            field_dict["options"] = options

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`Parser` from a dict"""
        d = src_dict.copy()
        item_types = []
        _item_types = d.pop("item_types")
        for item_types_item_data in _item_types:
            item_types_item = ItemType(item_types_item_data)

            item_types.append(item_types_item)

        name = d.pop("name")

        _options = d.pop("options", UNSET)
        options: Union[Unset, ParserOptions]
        if isinstance(_options, Unset):
            options = UNSET
        else:
            options = ParserOptions.from_dict(_options)

        parser = cls(
            item_types=item_types,
            name=name,
            options=options,
        )

        return parser



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="TaskDefRetryItem")


@_attrs_define
class TaskDefRetryItem:
    """TaskDefRetryItem model

    Attributes:
        cancel_after (Union[Unset, int]):  Default: 0.
        cancel_if (Union[Unset, str]):  Default: '$-'.
        fail_after (Union[Unset, int]):  Default: 0.
        fail_if (Union[Unset, str]):  Default: '$-'.
        max_retries (Union[Unset, int]):  Default: 1.
        priority_delta (Union[Unset, int]):  Default: 0.
        retry_after (Union[Unset, int]):  Default: 0.
        retry_if (Union[Unset, str]):  Default: '$-'.
    """

    cancel_after: Union[Unset, int] = 0
    cancel_if: Union[Unset, str] = "$-"
    fail_after: Union[Unset, int] = 0
    fail_if: Union[Unset, str] = "$-"
    max_retries: Union[Unset, int] = 1
    priority_delta: Union[Unset, int] = 0
    retry_after: Union[Unset, int] = 0
    retry_if: Union[Unset, str] = "$-"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        cancel_after = self.cancel_after
        cancel_if = self.cancel_if
        fail_after = self.fail_after
        fail_if = self.fail_if
        max_retries = self.max_retries
        priority_delta = self.priority_delta
        retry_after = self.retry_after
        retry_if = self.retry_if

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if cancel_after is not UNSET:
            field_dict["cancel_after"] = cancel_after
        if cancel_if is not UNSET:
            field_dict["cancel_if"] = cancel_if
        if fail_after is not UNSET:
            field_dict["fail_after"] = fail_after
        if fail_if is not UNSET:
            field_dict["fail_if"] = fail_if
        if max_retries is not UNSET:
            field_dict["max_retries"] = max_retries
        if priority_delta is not UNSET:
            field_dict["priority_delta"] = priority_delta
        if retry_after is not UNSET:
            field_dict["retry_after"] = retry_after
        if retry_if is not UNSET:
            field_dict["retry_if"] = retry_if

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`TaskDefRetryItem` from a dict"""
        d = src_dict.copy()
        cancel_after = d.pop("cancel_after", UNSET)

        cancel_if = d.pop("cancel_if", UNSET)

        fail_after = d.pop("fail_after", UNSET)

        fail_if = d.pop("fail_if", UNSET)

        max_retries = d.pop("max_retries", UNSET)

        priority_delta = d.pop("priority_delta", UNSET)

        retry_after = d.pop("retry_after", UNSET)

        retry_if = d.pop("retry_if", UNSET)

        task_def_retry_item = cls(
            cancel_after=cancel_after,
            cancel_if=cancel_if,
            fail_after=fail_after,
            fail_if=fail_if,
            max_retries=max_retries,
            priority_delta=priority_delta,
            retry_after=retry_after,
            retry_if=retry_if,
        )

        task_def_retry_item.additional_properties = d
        return task_def_retry_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from pypigeon.pigeon_core.api.admin import admin_tasks_results
from pypigeon.pigeon_core.api.admin import admin_tasks_schedules
from pypigeon.pigeon_core.api.admin import admin_tasks_stats

from .base_commands import BaseCommands


class TasksCommands(BaseCommands):
    """Queries against the task backend"""

    def stats(self) -> None:
        """Retrieve task statistics.

        Admin access is required.

        """
        tasks_stats = admin_tasks_stats.sync(client=self.core)

        self._output(tasks_stats.to_dict())

    def schedules(self) -> None:
        """Retrieve scheduled tasks.

        Admin access is required.

        """
        tasks_schedules = admin_tasks_schedules.sync(client=self.core)

        self._output(tasks_schedules.to_dict())

    @BaseCommands._with_arg("--id", help="task UUID")
    @BaseCommands._with_arg("--idemkey", help="task idemkey or schedule name")
    def results(self) -> None:
        """Retrieve results for a provided task ID or idemkey.

        A task schedule name can be provided as an idemkey to retrieve
        results from that scheduled task.

        Admin access is required.

        """
        if self.args.id:
            results = admin_tasks_results.sync(task_id=self.args.id, client=self.core)
        elif self.args.idemkey:
            results = admin_tasks_results.sync(
                task_idemkey=self.args.idemkey, client=self.core
            )
        else:
            raise ValueError("must provide either --id or --idemkey")

        self._output(results.to_dict())



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.admin_grants import AdminGrants


T = TypeVar("T", bound="AdminGrantAdminAdminListRequest")


@_attrs_define
class AdminGrantAdminAdminListRequest:
    """AdminGrantAdminAdminListRequest model

    Attributes:
        grants (List['AdminGrants']):
    """

    grants: List["AdminGrants"]

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        grants = []
        for grants_item_data in self.grants:
            grants_item = grants_item_data.to_dict()
            grants.append(grants_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "grants": grants,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`AdminGrantAdminAdminListRequest` from a dict"""
        d = src_dict.copy()
        grants = []
        _grants = d.pop("grants")
        for grants_item_data in _grants:
            grants_item = AdminGrants.from_dict(grants_item_data)

            grants.append(grants_item)

        admin_grant_admin_admin_list_request = cls(
            grants=grants,
        )

        return admin_grant_admin_admin_list_request



from typing import Any
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.admin_operations_item import AdminOperationsItem


T = TypeVar("T", bound="AdminGrants")


@_attrs_define
class AdminGrants:
    """AdminGrants model

    Attributes:
        operations (List[AdminOperationsItem]):
        subject_id (str):
    """

    operations: List[AdminOperationsItem]
    subject_id: str

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        operations = []
        for componentsschemasadmin_operations_item_data in self.operations:
            componentsschemasadmin_operations_item = (
                componentsschemasadmin_operations_item_data.value
            )
            operations.append(componentsschemasadmin_operations_item)

        subject_id = self.subject_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "operations": operations,
                "subject_id": subject_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`AdminGrants` from a dict"""
        d = src_dict.copy()
        operations = []
        _operations = d.pop("operations")
        for componentsschemasadmin_operations_item_data in _operations:
            componentsschemasadmin_operations_item = AdminOperationsItem(
                componentsschemasadmin_operations_item_data
            )

            operations.append(componentsschemasadmin_operations_item)

        subject_id = d.pop("subject_id")

        admin_grants = cls(
            operations=operations,
            subject_id=subject_id,
        )

        return admin_grants



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar

from attrs import define as _attrs_define

from ..models.data_element_permissible_values_number_range_number_range import (
    DataElementPermissibleValuesNumberRangeNumberRange,
)


T = TypeVar("T", bound="DataElementPermissibleValuesNumberRange")


@_attrs_define
class DataElementPermissibleValuesNumberRange:
    """DataElementPermissibleValuesNumberRange model

    Attributes:
        number_range (DataElementPermissibleValuesNumberRangeNumberRange): Controls on the numerical domain of the data.
    """

    number_range: "DataElementPermissibleValuesNumberRangeNumberRange"

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        number_range = self.number_range.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "numberRange": number_range,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`DataElementPermissibleValuesNumberRange` from a dict"""
        d = src_dict.copy()
        number_range = DataElementPermissibleValuesNumberRangeNumberRange.from_dict(
            d.pop("numberRange")
        )

        data_element_permissible_values_number_range = cls(
            number_range=number_range,
        )

        return data_element_permissible_values_number_range



"""Copy an item into this collection"""
from http import HTTPStatus
from typing import Any
from typing import Dict
from typing import Union

import httpx

from ... import errors
from ...client import AuthenticatedClient
from ...client import Client
from ...models.collections_items_copy_item_body import CollectionsItemsCopyItemBody
from ...models.item import Item
from ...models.server_error import ServerError
from ...types import Response


def _get_kwargs(
    collection_id: str,
    version_id: str,
    *,
    body: CollectionsItemsCopyItemBody,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}

    _kwargs: Dict[str, Any] = {
        "method": "post",
        "url": "/collections/{collection_id}/{version_id}/items/copy".format(
            collection_id=collection_id,
            version_id=version_id,
        ),
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Union[Item, ServerError]:
    if response.status_code == HTTPStatus.CREATED:
        response_201 = Item.from_dict(response.json())

        return response_201
    if response.status_code == HTTPStatus.BAD_REQUEST:
        response_400 = ServerError.from_dict(response.json())

        return response_400
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = ServerError.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = ServerError.from_dict(response.json())

        return response_403
    if response.status_code == HTTPStatus.CONFLICT:
        response_409 = ServerError.from_dict(response.json())

        return response_409
    if response.status_code == HTTPStatus.UNSUPPORTED_MEDIA_TYPE:
        response_415 = ServerError.from_dict(response.json())

        return response_415
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = ServerError.from_dict(response.json())

        return response_500

    raise errors.UnexpectedStatus(response.status_code, response.content)


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Item, ServerError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    collection_id: str,
    version_id: str,
    *,
    client: AuthenticatedClient,
    body: CollectionsItemsCopyItemBody,
) -> Response[Union[Item, ServerError]]:
    """Copy an item into this collection

    Copy an item from a local or remote source into this collection.

    The source item must be specified as a pigeon:// URL, and the
    user creating the copy must have read access to the source
    item. The new item name can be specified, along with the
    destination folder; all other metadata changes must be done
    once the item copy is complete.

    All item types support being copied:

    * Copied file and dataviews will share the same
      pigeon+datastore:// target, preventing a duplicate copy of
      the data from being stored in the datastore. Copies will not
      affect each other, though, as all writes to file items
      result in a new pigeon+datastore:// URL.

    * Links will copy their storage target URLs, and no data will
      be copied.

    * Folders can be copied, however, their contents are not
      recursively copied. To copy an entire folder structure,
      clients should follow a recursive breadth-first algorithm to
      copy the folder, followed by its contents.

    * Dataviews can be copied, however, their data sources will
      not automatically map to new data sources if they are copied
      to a new collection. Clients should assist users to remap
      dataview sources if/when those sources are broken.

    Args:
        collection_id (str):
        version_id (str):
        body (CollectionsItemsCopyItemBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Item, ServerError]]
    """

    kwargs = _get_kwargs(
        collection_id=collection_id,
        version_id=version_id,
        body=body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    collection_id: str,
    version_id: str,
    *,
    client: AuthenticatedClient,
    body: CollectionsItemsCopyItemBody,
) -> Union[Item]:
    """Copy an item into this collection

    Copy an item from a local or remote source into this collection.

    The source item must be specified as a pigeon:// URL, and the
    user creating the copy must have read access to the source
    item. The new item name can be specified, along with the
    destination folder; all other metadata changes must be done
    once the item copy is complete.

    All item types support being copied:

    * Copied file and dataviews will share the same
      pigeon+datastore:// target, preventing a duplicate copy of
      the data from being stored in the datastore. Copies will not
      affect each other, though, as all writes to file items
      result in a new pigeon+datastore:// URL.

    * Links will copy their storage target URLs, and no data will
      be copied.

    * Folders can be copied, however, their contents are not
      recursively copied. To copy an entire folder structure,
      clients should follow a recursive breadth-first algorithm to
      copy the folder, followed by its contents.

    * Dataviews can be copied, however, their data sources will
      not automatically map to new data sources if they are copied
      to a new collection. Clients should assist users to remap
      dataview sources if/when those sources are broken.

    Args:
        collection_id (str):
        version_id (str):
        body (CollectionsItemsCopyItemBody):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Item]
    """

    response = sync_detailed(
        collection_id=collection_id,
        version_id=version_id,
        client=client,
        body=body,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed


async def asyncio_detailed(
    collection_id: str,
    version_id: str,
    *,
    client: AuthenticatedClient,
    body: CollectionsItemsCopyItemBody,
) -> Response[Union[Item, ServerError]]:
    """Copy an item into this collection

    Copy an item from a local or remote source into this collection.

    The source item must be specified as a pigeon:// URL, and the
    user creating the copy must have read access to the source
    item. The new item name can be specified, along with the
    destination folder; all other metadata changes must be done
    once the item copy is complete.

    All item types support being copied:

    * Copied file and dataviews will share the same
      pigeon+datastore:// target, preventing a duplicate copy of
      the data from being stored in the datastore. Copies will not
      affect each other, though, as all writes to file items
      result in a new pigeon+datastore:// URL.

    * Links will copy their storage target URLs, and no data will
      be copied.

    * Folders can be copied, however, their contents are not
      recursively copied. To copy an entire folder structure,
      clients should follow a recursive breadth-first algorithm to
      copy the folder, followed by its contents.

    * Dataviews can be copied, however, their data sources will
      not automatically map to new data sources if they are copied
      to a new collection. Clients should assist users to remap
      dataview sources if/when those sources are broken.

    Args:
        collection_id (str):
        version_id (str):
        body (CollectionsItemsCopyItemBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Item, ServerError]]
    """

    kwargs = _get_kwargs(
        collection_id=collection_id,
        version_id=version_id,
        body=body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    collection_id: str,
    version_id: str,
    *,
    client: AuthenticatedClient,
    body: CollectionsItemsCopyItemBody,
) -> Union[Item]:
    """Copy an item into this collection

    Copy an item from a local or remote source into this collection.

    The source item must be specified as a pigeon:// URL, and the
    user creating the copy must have read access to the source
    item. The new item name can be specified, along with the
    destination folder; all other metadata changes must be done
    once the item copy is complete.

    All item types support being copied:

    * Copied file and dataviews will share the same
      pigeon+datastore:// target, preventing a duplicate copy of
      the data from being stored in the datastore. Copies will not
      affect each other, though, as all writes to file items
      result in a new pigeon+datastore:// URL.

    * Links will copy their storage target URLs, and no data will
      be copied.

    * Folders can be copied, however, their contents are not
      recursively copied. To copy an entire folder structure,
      clients should follow a recursive breadth-first algorithm to
      copy the folder, followed by its contents.

    * Dataviews can be copied, however, their data sources will
      not automatically map to new data sources if they are copied
      to a new collection. Clients should assist users to remap
      dataview sources if/when those sources are broken.

    Args:
        collection_id (str):
        version_id (str):
        body (CollectionsItemsCopyItemBody):

    Raises:
        errors.UnexpectedStatus: If the server returns a status code greater than or equal to 300.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Item]
    """

    response = await asyncio_detailed(
        collection_id=collection_id,
        version_id=version_id,
        client=client,
        body=body,
    )
    if isinstance(response.parsed, ServerError):
        raise errors.UnexpectedStatus(response.status_code, response.content)

    return response.parsed



from typing import Any
from typing import Dict
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.update_collection_metadata import UpdateCollectionMetadata
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="UpdateCollection")


@_attrs_define
class UpdateCollection:
    """Update core collection properties

    Attributes:
        account_id (Union[Unset, str]): The account into which the collection should be moved
        description (Union[Unset, str]): The collection's full description
        metadata (Union[Unset, UpdateCollectionMetadata]): Arbitrary user-defined metadata, key-value pairs
        name (Union[Unset, str]): The collection's short human-readable name
    """

    account_id: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    metadata: Union[Unset, "UpdateCollectionMetadata"] = UNSET
    name: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        account_id = self.account_id
        description = self.description
        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()
        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if account_id is not UNSET:
            field_dict["account_id"] = account_id
        if description is not UNSET:
            field_dict["description"] = description
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`UpdateCollection` from a dict"""
        d = src_dict.copy()
        account_id = d.pop("account_id", UNSET)

        description = d.pop("description", UNSET)

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, UpdateCollectionMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = UpdateCollectionMetadata.from_dict(_metadata)

        name = d.pop("name", UNSET)

        update_collection = cls(
            account_id=account_id,
            description=description,
            metadata=metadata,
            name=name,
        )

        return update_collection



from typing import Any
from typing import cast
from typing import Dict
from typing import List
from typing import Type
from typing import TypeVar
from typing import Union

from attrs import define as _attrs_define

from ..models.collections_items_create_item_json_new_item_request_content import (
    CollectionsItemsCreateItemJsonNewItemRequestContent,
)
from ..models.item_column import ItemColumn
from ..models.item_metadata import ItemMetadata
from ..models.item_parser import ItemParser
from ..models.item_storage import ItemStorage
from ..models.item_type import ItemType
from ..types import UNSET
from ..types import Unset


T = TypeVar("T", bound="CollectionsItemsCreateItemJsonNewItemRequest")


@_attrs_define
class CollectionsItemsCreateItemJsonNewItemRequest:
    """CollectionsItemsCreateItemJsonNewItemRequest model

    Attributes:
        name (str): Item name
        type (ItemType):
        columns (Union[List['ItemColumn'], None, Unset]):
        content (Union[Unset, CollectionsItemsCreateItemJsonNewItemRequestContent]): If uploading a small file, the
            contents can be provided.
        folder_id (Union[Unset, str]): Item ID of parent folder, or ROOT Example: ROOT.
        metadata (Union[Unset, ItemMetadata]): Arbitrary metadata as key-value string pairs
        parser (Union['ItemParser', None, Unset]):  Example: {'name': 'auto'}.
        storage (Union['ItemStorage', None, Unset]):  Example: {'checksum': {}, 'size': None, 'sizeIsEstimate': False,
            'url': 'pigeon+datastore://default'}.
    """

    name: str
    type: ItemType
    columns: Union[List["ItemColumn"], None, Unset] = UNSET
    content: Union[Unset, "CollectionsItemsCreateItemJsonNewItemRequestContent"] = UNSET
    folder_id: Union[Unset, str] = UNSET
    metadata: Union[Unset, "ItemMetadata"] = UNSET
    parser: Union["ItemParser", None, Unset] = UNSET
    storage: Union["ItemStorage", None, Unset] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        """Convert to a dict"""
        name = self.name
        type = self.type.value
        columns: Union[List[Dict[str, Any]], None, Unset]
        if isinstance(self.columns, Unset):
            columns = UNSET
        elif isinstance(self.columns, list):
            columns = []
            for componentsschemasitem_columns_type_0_item_data in self.columns:
                componentsschemasitem_columns_type_0_item = (
                    componentsschemasitem_columns_type_0_item_data.to_dict()
                )
                columns.append(componentsschemasitem_columns_type_0_item)

        else:
            columns = self.columns
        content: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.content, Unset):
            content = self.content.to_dict()
        folder_id = self.folder_id
        metadata: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.metadata, Unset):
            metadata = self.metadata.to_dict()
        parser: Union[Dict[str, Any], None, Unset]
        if isinstance(self.parser, Unset):
            parser = UNSET
        elif isinstance(self.parser, ItemParser):
            parser = self.parser.to_dict()
        else:
            parser = self.parser
        storage: Union[Dict[str, Any], None, Unset]
        if isinstance(self.storage, Unset):
            storage = UNSET
        elif isinstance(self.storage, ItemStorage):
            storage = self.storage.to_dict()
        else:
            storage = self.storage

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
                "type": type,
            }
        )
        if columns is not UNSET:
            field_dict["columns"] = columns
        if content is not UNSET:
            field_dict["content"] = content
        if folder_id is not UNSET:
            field_dict["folderId"] = folder_id
        if metadata is not UNSET:
            field_dict["metadata"] = metadata
        if parser is not UNSET:
            field_dict["parser"] = parser
        if storage is not UNSET:
            field_dict["storage"] = storage

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        """Create an instance of :py:class:`CollectionsItemsCreateItemJsonNewItemRequest` from a dict"""
        d = src_dict.copy()
        name = d.pop("name")

        type = ItemType(d.pop("type"))

        def _parse_columns(data: object) -> Union[List["ItemColumn"], None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                componentsschemasitem_columns_type_0 = []
                _componentsschemasitem_columns_type_0 = data
                for (
                    componentsschemasitem_columns_type_0_item_data
                ) in _componentsschemasitem_columns_type_0:
                    componentsschemasitem_columns_type_0_item = ItemColumn.from_dict(
                        componentsschemasitem_columns_type_0_item_data
                    )

                    componentsschemasitem_columns_type_0.append(
                        componentsschemasitem_columns_type_0_item
                    )

                return componentsschemasitem_columns_type_0
            except:  # noqa: E722
                pass
            return cast(Union[List["ItemColumn"], None, Unset], data)

        columns = _parse_columns(d.pop("columns", UNSET))

        _content = d.pop("content", UNSET)
        content: Union[Unset, CollectionsItemsCreateItemJsonNewItemRequestContent]
        if isinstance(_content, Unset):
            content = UNSET
        else:
            content = CollectionsItemsCreateItemJsonNewItemRequestContent.from_dict(
                _content
            )

        folder_id = d.pop("folderId", UNSET)

        _metadata = d.pop("metadata", UNSET)
        metadata: Union[Unset, ItemMetadata]
        if isinstance(_metadata, Unset):
            metadata = UNSET
        else:
            metadata = ItemMetadata.from_dict(_metadata)

        def _parse_parser(data: object) -> Union["ItemParser", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemasitem_parser_type_0 = ItemParser.from_dict(data)

                return componentsschemasitem_parser_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ItemParser", None, Unset], data)

        parser = _parse_parser(d.pop("parser", UNSET))

        def _parse_storage(data: object) -> Union["ItemStorage", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemasitem_storage_type_0 = ItemStorage.from_dict(data)

                return componentsschemasitem_storage_type_0
            except:  # noqa: E722
                pass
            return cast(Union["ItemStorage", None, Unset], data)

        storage = _parse_storage(d.pop("storage", UNSET))

        collections_items_create_item_json_new_item_request = cls(
            name=name,
            type=type,
            columns=columns,
            content=content,
            folder_id=folder_id,
            metadata=metadata,
            parser=parser,
            storage=storage,
        )

        return collections_items_create_item_json_new_item_request
